/* 
 * A simple demonstration of the Student structure with linked lists
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#define ID_LENGTH 9
#define NAME_LENGTH 32

typedef struct {
	char id[ID_LENGTH+1];
	char name[NAME_LENGTH+1];
	int  grade;

    /* pointer to the next link in the list */
	struct student *next;
} sStudent;


/* Free a list of students */
void free_list(sStudent *head)
{
	if (head != NULL)
	{
		free_list(head->next);
		free(head);
	}
}

/* Allocate a new student */
sStudent* new_student(char *name, char* id, int grade)
{
	sStudent *std = NULL;

	std = (sStudent*)malloc(sizeof(sStudent));
	if (std == NULL) 
	{
		printf("Memory allocation error!\n");
		return NULL;
	}

	strcpy(std->name, name);
	strcpy(std->id, id);
	std->grade = grade;
	std->next = NULL;

	return std;
}


/*
 * Insert a student to the list, keeping the list sorted in
 * descending order of the grades.
 */
sStudent *add_student(sStudent *head, sStudent *std)
{
	sStudent *next_std = NULL,
             *prev_std = NULL;

	if (head == NULL) /* The list is empty */
		return std;

	next_std = head;
	while (std->grade < next_std->grade)
	{
		if (next_std->next == NULL) /* std has the lowest grade */
		{
			next_std->next = std;
			return head;
		}
		
		prev_std = next_std;
		next_std = next_std->next;
	}

	/* If we got here, next should be the next element after std, 
	   prev should be the one before */
	if (prev_std == NULL) /* Our student has the highest grade. */
	{
		/* Make head out of it */
		std->next = head; 
		return std;
	}
	
	prev_std->next = std;
	std->next = next_std;

	return head;
}

void print_list(sStudent *list)
{
	while (list != NULL)
	{
		printf("Name: %s; ID: %s; Grade: %d\n", list->name, list->id, list->grade);
		list = list->next;
	}
}


/* find a student whose id matches the given id number */
sStudent *find_student(sStudent *head, char *id)
{
	/* YOUR CODE GOES HERE */
}


int main()
{
	int i=0, grade = 0;
	sStudent *head = NULL, *std = NULL, *ptr = NULL;
	char name[100] = { 0 }, id[10] = { 0 };

	printf("Please enter four students\n");

	for (i=0; i < 4; i += 1)
	{
		printf("Name: (less than 100 characters)\n");
		gets( name );

		printf("Id: (9 characters)\n");
		scanf("%9s", id );

		printf("Grade:\n");
		scanf("%d", &grade);

		/* Allocate a student */
		ptr = new_student(name, id, grade);
		if (ptr == NULL)
		{
			free_list(head);
			return -1;
		}
		/* Add the student */
		head = add_student(head, ptr);

		scanf("%*c");
	}

	print_list(head);

	printf("Enter the id of the student you wish to find\n");
	scanf("%9s", id);

	std = find_student(head, id);
	if (std != NULL)
		printf("\nThe student whose id is %s is - %s\n", std->id, std->name);
	else
		printf("No student exists with id %s\n", id);

	free_list(head);

	return 0;
}
